#include <fstream>
#include <iostream>

#include "vtkExport.h"

/**
 * Musterloesung fuer Programmieraufgabe 3:
 * Approximiert 4 Bezierpatches, jeweils in der Aufloesung 10x10 (hartkodiert).
 * Als Ergebnis erhaelt man eine Vase; die Ausgabedatei "vase.vtu" kann
 * beispielsweise mit Paraview geoeffnet und visualisiert werden.
 **/

// Bildet Knotenposition (i, j) auf den zugehoerigen Index im Knotenvektor ab
inline unsigned int ijmap(unsigned int i, unsigned int j) {
  return j*10 + i;
}

int main () {
  typedef std::vector<std::vector<double> > VecVecRType;
  typedef std::vector<std::vector<unsigned int> > VecVecIType;

  std::ifstream in ( "patch.txt" );

  VecVecRType points(400);
  VecVecIType elements;
  for(unsigned int i = 0; i < points.size(); ++i)
    points[i].resize(3);

  // Lese Patches aus Eingabedatei
  for ( int patch = 0; patch < 4; ++patch ) {
    // Offset fuer den Knotenvektor, Knoten fuer Patches werden nacheinander abgespeichert
    unsigned int offset = patch*100;

    double p[4][4][3];

    for ( int i = 0; i < 4; ++i )
      for ( int j = 0; j < 4; ++j )
        for ( int k = 0; k < 3; ++k )
          in >> p[i][j][k];


    // Berechne Knotepositionen
    for (int i = 0; i < 10; ++i) {
      for (int j = 0; j < 10; ++j) {
        double lambda [2] = { static_cast<double> ( i ) / 9, static_cast<double> ( j ) / 9 };
        double ilambda[2] = { 1 - lambda [0], 1 - lambda [1] };

        // pp Array fuer Groessen b_{ij}^k
        double pp[4][4][3];

        // Initialisiere b_{ij}^0
        for ( int a = 0; a < 4; ++a )
          for ( int b = 0; b < 4; ++b )
            for ( int c = 0; c < 3; ++c )
              pp[a][b][c] = p[a][b][c];

        // De Cateljau fuer Tensorprodukt-Bezierpolynome
        for ( int a = 0; a < 3; ++a )
          for ( int b = 0; b < 3-a; ++b )
            for ( int c = 0; c < 3-a; ++c )
              for ( int d = 0; d < 3; ++d )
                pp[b][c][d] =  lambda[0] * lambda[1] * pp[b][c][d]
                               + lambda[0] * ilambda[1] * pp[b][c+1][d]
                               + ilambda[0] * lambda[1] * pp[b+1][c][d]
                               + ilambda[0] * ilambda[1] * pp[b+1][c+1][d];

        // Schreibe Ergebnis nach points
        for ( int c = 0; c < 3; ++c )
          points[offset + ijmap(i, j)][c] = pp[0][0][c];
      }
    }

    // Angrenzende Punkte bilden ein Viereck
    for(int i = 1; i < 10; ++i) {
      for(int j = 1; j < 10; ++j) {
        elements.push_back(std::vector<unsigned int> {offset + (j - 1)*10 + (i - 1),
                           offset + (j - 1)*10 + i,
                           offset + j*10 + i,
                           offset + j*10 + (i - 1)});
      }
    }
  }

  // Exportiere das Ergebnis
  exportGrid(points, elements, "vase.vtu");

  return 0;
}
